@@ -3,7 +3,7 @@ use strict;
 
 package JMAP::Tester::Result::Auth;
 # ABSTRACT: what you get when you authenticate
-$JMAP::Tester::Result::Auth::VERSION = '0.011';
+$JMAP::Tester::Result::Auth::VERSION = '0.012';
 use Moo;
 with 'JMAP::Tester::Role::Result';
 
@@ -18,7 +18,7 @@ use namespace::clean;
 
 sub is_success { 1 }
 
-has auth_struct => (
+has client_session => (
   is => 'ro',
 );
 
@@ -36,7 +36,7 @@ JMAP::Tester::Result::Auth - what you get when you authenticate
 
 =head1 VERSION
 
-version 0.011
+version 0.012
 
 =head1 OVERVIEW
 
